#version 330 core

const int NUM_HEXES = 812;

in vec2 texCoord;
in vec2 texCoord1;
in vec3 normal;
in vec3 indizes;
in vec3 lightDir;
in vec3 viewDir;
in vec3 halfVec;
in vec4 shadowCoord;

layout(location = 0)out vec4 fragColor;

uniform sampler2D shadow;
uniform sampler2D grid;
uniform sampler2D tiles;
uniform sampler2D tiles_normals;
uniform sampler2D border;
uniform samplerBuffer visibilities;
uniform samplerBuffer owner;

layout(std140) uniform LightingInfo
{
	vec3 lightPosition;
	vec3 ambientColor;
	vec3 lightColor;
};

uniform int isSelection;


vec4 phongModel(vec3 norm, float spec, vec3 diffR, float vis)
{
	vec3 ambient = ambientColor * lightColor;	
	float sDotN = clamp( dot(lightDir, norm), 0.0,1.0);
	vec3 diffuse = lightColor * sDotN;
	
	vec3 s = vec3(0.0);
	if(sDotN > 0.0)
	{
		s = lightColor *0.8 * pow( clamp( dot(halfVec,norm)*spec*1.3, 0.0, 1.0),10);
	}
	return vec4((ambient*diffR) + (( diffuse)*diffR + s)*vis,sDotN);
}

float offsetLookup(vec2 uv, vec2 offset)
{
	return texture(shadow, uv+offset*1.0/2048.0).x;
}

float pcf(vec2 uv, float d, float bias)
{
	float vis = 1.0f;
	for(float x = -1.5; x<=1.5; x++)
	{
		for(float y = -1.5; y<=1.5; y++)
		{
			if(offsetLookup(uv,vec2(x,y)) < d-bias)
			{
				vis -= 1.0/16.0;
			}
		}
	}
	return vis;
}

int getIndex(vec4 gridColor)
{
	int index = int(indizes.x);
	if(gridColor.x > 0.5)
		index = int(indizes.z);
	else if(gridColor.y>0.5)
		index = int(indizes.y);
	return index;
}


vec3 getAtmosphereColor(vec3 norm, vec3 sun, float rimSize, float rimStrength, float atmosphereSize, float atmosphereStrength)
{
	vec3 col = vec3(1,0.98,0.93);

	float r = min(1,rimSize*length(norm.xy));
	r = r*r*r*r*r;
	vec3 rimColor = col * r * rimStrength;

	vec3 n = norm;
	n.z *= -1.1;
	n = normalize(n);
	float s = max(0,dot(n,normalize(lightPosition)));
	s = min( 1, atmosphereSize*s*s*s*s*s);
	vec3 atmo = col*s*atmosphereStrength*r;

	return rimColor + atmo;
}

vec4 getBorder(int index)
{
	int i1 = int(indizes.x);
	int i2 = int(indizes.y);
	int i3 = int(indizes.z);

	float o1 = texelFetch(owner, i1).r;
	float o2 = texelFetch(owner, i2).r;
	float o3 = texelFetch(owner, i3).r;

	if(o1 == o2 && o2 == o3)
		return vec4(0);
	vec4 ret = vec4(0);
	vec4 tex = texture(border, texCoord);
	vec3 col[2];
	col[0] = vec3(1,0,0);
	col[1] = vec3(0,0,1);
	if((index == i1 && o1>0))
	{
		
		if(o1 != o2 && o1 != o3)
			ret = vec4(col[int(o1)-1],tex.r);
		else if(o1 == o2)
			ret = vec4(col[int(o1)-1],tex.g);
		else if(o1 == o3)
			ret = vec4(col[int(o1)-1],tex.b);
	}else if(index == i2 && o2>0)
	{
		if(o2 != o3 && o2 != o1)
			ret = vec4(col[int(o2)-1],tex.r);
		else if(o2 == o1)
			ret = vec4(col[int(o2)-1],tex.g);
		else if( o2 == o3)
			ret = vec4(col[int(o2)-1],tex.a);
	}else if(index == i3 && o3>0)
	{
		if(o3 != o1 && o3 != o2)
			ret = vec4(col[int(o3)-1],tex.r);
		else if(o3 == o1)
			ret = vec4(col[int(o3)-1],tex.b);
		else if(o3 == o2)
			ret = vec4(col[int(o3)-1],tex.a);
	}
	return ret;
}

void main()
{	
	if(isSelection > 0)
	{
		vec3 c = vec3(0,1,0);
		if(isSelection == 2)
			c = vec3(1,0,0);
		float s = length(texCoord.xy)*0.5;
		s = s*s*s*s;
		fragColor=vec4(s*c,1);
	}
	else
	{
		vec4 gridColor = texture(grid,texCoord);
		vec4 diffuse = texture(tiles,texCoord1);
		vec4 normalAndSpec = texture(tiles_normals,texCoord1);
		vec3 norm = normalAndSpec.xyz*2.0 - 1.0;
		float spec = normalAndSpec.w;

		int index = getIndex(gridColor);
		float vis = texelFetch(visibilities, index).r;
	


		float shad = pcf(shadowCoord.xy, shadowCoord.z,0.01f);
		vec4 col = phongModel(norm, spec, diffuse.rgb, shad);
		vec4 b = getBorder(index);
		b.w = b.w*0.4 + b.w*0.6*col.w;
		if(vis<=0.2)
			b = vec4(0);
		col = col*vis;


		vec3 atmosphere = getAtmosphereColor(normal, lightPosition,0.9,0.7,1,1);
		fragColor=vec4(mix(atmosphere+col.xyz,b.xyz,b.w*0.6),1);
	}
}